@@ -80,13 +80,26 @@ static atom_t ATOM_error;
 static atom_t ATOM_fatal;
 static atom_t ATOM_clear;
 static atom_t ATOM_setsize;
+static atom_t ATOM_getsize;
 static atom_t ATOM_setunique;
+static atom_t ATOM_set;
+static atom_t ATOM_prev_str;
+static atom_t ATOM_next_str;
+static atom_t ATOM_event;
 
+static functor_t FUNCTOR_error2;
+static functor_t FUNCTOR_editline1;
 static functor_t FUNCTOR_line2;
 static functor_t FUNCTOR_electric3;
 static functor_t FUNCTOR_pair2;
 
-#define STR_OPTIONS (CVT_ATOM|CVT_STRING|CVT_LIST|REP_MB|CVT_EXCEPTION)
+#ifdef __WINDOWS__
+#define REP_EL REP_UTF8
+#else
+#define REP_EL REP_MB
+#endif
+
+#define STR_OPTIONS (CVT_ATOM|CVT_STRING|CVT_LIST|REP_EL|CVT_EXCEPTION)
 
 		 /*******************************
 		 *           WIN/UNIX           *
@@ -157,7 +170,9 @@ typedef struct el_context
   command	       *commands;	/* User commands */
   binding	       *bindings;	/* Bindings to user commands */
   int			reader;		/* Current reader thread */
+  bool			is_stdin;	/* Reading from file 0 */
   unsigned int		flags;		/* Misc flags */
+  int			histsize;	/* History size */
   struct
   { int timeout;			/* Time to wait */
     int move;				/* Amount to move */
@@ -238,6 +253,18 @@ update_prompt(el_context *ctx)
 }
 
 
+static bool
+el_error(const char *msg)
+{ term_t ex;
+
+  return ( (ex=PL_new_term_ref()) &&
+	   PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
+			       PL_FUNCTOR, FUNCTOR_editline1,
+				 PL_STRING, msg,
+			       PL_VARIABLE) &&
+	   PL_raise_exception(ex) );
+}
+
 		 /*******************************
 		 *		PORT		*
 		 *******************************/
@@ -397,11 +424,11 @@ el_sighandler(int sig)
 
 static const char *
 el_siggets(EditLine *el, int *count)
-{ const char *line;
-  FILE *in;
+{ el_context *ctx;
+  const char *line;
 
-  el_get(el, EL_GETFP, 0, &in);
-  if ( fileno(in) == 0 )
+  el_get(el, EL_CLIENTDATA, (void**)&ctx);
+  if ( ctx->is_stdin )
   { prepare_signals(el_signals);
     line = el_gets(el, count);
     restore_signals(el_signals);
@@ -1200,6 +1227,7 @@ prompt(EditLine *el)
 
 static PL_option_t el_wrap_options[] =
 { PL_OPTION("pipes",   OPT_BOOL),
+  PL_OPTION("history", OPT_INT),
   PL_OPTIONS_END
 };
 
@@ -1210,11 +1238,12 @@ pl_wrap(term_t progid, term_t tin, term_t tout, term_t terr, term_t options)
   char *prog;
   unsigned int el_flags = 0;
   int pipes = false;
+  int histsize = 100;
 
   if ( !PL_get_chars(progid, &prog, STR_OPTIONS) )
     return FALSE;
   if ( !PL_scan_options(options, 0, "el_wrap_options",
-			el_wrap_options, &pipes) )
+			el_wrap_options, &pipes, &histsize) )
     return FALSE;
   if ( pipes )
     el_flags |= EPILOG;
@@ -1244,6 +1273,7 @@ pl_wrap(term_t progid, term_t tin, term_t tout, term_t terr, term_t options)
 	setlinebuf(fin);
 	setlinebuf(fout);
 	setbuf(ferr, NULL);
+	ctx->is_stdin = (fd_in == 0);
 #endif
 	ctx->flags   = el_flags;
 	ctx->istream = in;
@@ -1251,7 +1281,8 @@ pl_wrap(term_t progid, term_t tin, term_t tout, term_t terr, term_t options)
 	ctx->estream = err;
 
 	ctx->history = history_init();
-	history(ctx->history, &ctx->ev, H_SETSIZE,   100);
+	ctx->histsize = histsize;
+	history(ctx->history, &ctx->ev, H_SETSIZE,   histsize);
 	history(ctx->history, &ctx->ev, H_SETUNIQUE, TRUE);
 
 #ifdef __WINDOWS__
@@ -1839,7 +1870,7 @@ pl_add_history(term_t tin, term_t text)
   char *line;
 
 
-  if ( !PL_get_chars(text, &line, CVT_ATOM|CVT_STRING|REP_UTF8|CVT_EXCEPTION) ||
+  if ( !PL_get_chars(text, &line, STR_OPTIONS) ||
        !get_el_context(tin, &ctx) )
     return FALSE;
 
@@ -1874,11 +1905,12 @@ pl_read_history(term_t tin, term_t file_name)
        PL_get_file_name(file_name, &fname,
 			PL_FILE_OSPATH|PL_FILE_SEARCH|PL_FILE_READ|
 			PL_FILE_NOERRORS) )
-  { history(ctx->history, &ctx->ev, H_LOAD, fname);
-    return TRUE;
+  { if ( history(ctx->history, &ctx->ev, H_LOAD, fname) < 0 )
+      return el_error(ctx->ev.str);
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1925,31 +1957,80 @@ pl_history_events(term_t tin, term_t events)
 }
 
 
-static int
+static bool
 get_int_arg(int i, term_t t, int *v)
 { term_t a;
 
-  if ( (a=PL_new_term_ref()) &&
-       PL_get_arg(i, t, a) &&
-       PL_get_integer_ex(a, v) )
-    return TRUE;
-
-  return FALSE;
+  return ( (a=PL_new_term_ref()) &&
+	   PL_get_arg(i, t, a) &&
+	   PL_get_integer_ex(a, v) );
 }
 
-static int
+static bool
 get_bool_arg(int i, term_t t, int *v)
 { term_t a;
 
-  if ( (a=PL_new_term_ref()) &&
-       PL_get_arg(i, t, a) &&
-       PL_get_bool_ex(a, v) )
-    return TRUE;
+  return ( (a=PL_new_term_ref()) &&
+	   PL_get_arg(i, t, a) &&
+	   PL_get_bool_ex(a, v) );
+}
 
-  return FALSE;
+static bool
+get_str_arg(int i, term_t t, char **s)
+{ term_t a;
+
+  return ( (a=PL_new_term_ref()) &&
+	   PL_get_arg(i, t, a) &&
+	   PL_get_chars(a, s, STR_OPTIONS) );
+}
+
+static bool
+unify_int_arg(int i, term_t t, int v)
+{ term_t a;
+
+  return ( (a=PL_new_term_ref()) &&
+	   PL_get_arg(i, t, a) &&
+	   PL_unify_integer(a, v) );
+}
+
+static bool
+unify_str_arg(int i, term_t t, const char *str)
+{ term_t a;
+
+  return ( (a=PL_new_term_ref()) &&
+	   PL_get_arg(i, t, a) &&
+	   PL_unify_chars(a, PL_STRING|REP_EL, (size_t)-1, str) );
 }
 
 
+typedef struct
+{ const char *name;
+  int         code;
+  atom_t      atom;
+} q0_event;			/* Query event without argument */
+
+static q0_event q0_events[] = {
+  { "first", H_FIRST },
+  { "last",  H_LAST },
+  { "prev",  H_PREV },
+  { "next",  H_NEXT },
+  { "curr",  H_CURR },
+  { NULL,    0 }
+};
+
+static bool
+q0_action(atom_t key, int *action)
+{ for(q0_event *ev = q0_events; ev->name; ev++)
+  { if ( !ev->atom )
+      ev->atom = PL_new_atom(ev->name);
+    if ( ev->atom == key )
+    { *action = ev->code;
+      return true;
+    }
+  }
+
+  return false;
+}
 
 static foreign_t
 pl_history(term_t tin, term_t option)
@@ -1959,6 +2040,7 @@ pl_history(term_t tin, term_t option)
   { atom_t name;
     size_t arity;
     int rc = 0;
+    int code;
 
     if ( PL_get_name_arity(option, &name, &arity) )
     { HistEvent ev;
@@ -1968,6 +2050,48 @@ pl_history(term_t tin, term_t option)
 	if ( arity != 1 ) goto err_domain;
 	if ( !get_int_arg(1, option, &s) ) return FALSE;
 	rc = history(ctx->history, &ev, H_SETSIZE, s);
+	if ( rc == 0 )
+	  ctx->histsize = s;
+      } else if ( arity == 1 && name == ATOM_getsize )
+      { term_t a;
+
+	return ( (a = PL_new_term_ref()) &&
+		 PL_get_arg(1, option, a) &&
+		 PL_unify_integer(a, ctx->histsize) );
+      } else if ( arity == 2 && q0_action(name, &code) )
+      { if ( history(ctx->history, &ev, code) == 0 )
+	{ return ( unify_int_arg(1, option, ev.num) &&
+		   unify_str_arg(2, option, ev.str) );
+	} else
+	  return false;
+      } else if ( arity == 3 && (name == ATOM_prev_str ||
+				 name == ATOM_next_str) )
+      { char *s;
+	if ( !get_str_arg(1, option, &s) ) return false;
+	code = name == ATOM_prev_str ? H_PREV_STR : H_NEXT_STR;
+	if ( history(ctx->history, &ev, code, s) == 0 )
+	  return ( unify_int_arg(2, option, ev.num) &&
+		   unify_str_arg(3, option, ev.str) );
+	else
+	  return false;
+      } else if ( arity == 2 && name == ATOM_event )
+      { int i;
+
+	rc = false;
+	if ( !get_int_arg(1, option, &i) ) return false;
+	if ( history(ctx->history, &ev, H_CURR) == 0 )
+	{ int curr = ev.num;
+	  if ( history(ctx->history, &ev, H_SET, i) == 0 )
+	  { if ( history(ctx->history, &ev, H_CURR) == 0 )
+	      rc = unify_str_arg(2, option, ev.str);
+	  }
+	  history(ctx->history, &ev, H_SET, curr);
+	}
+	return rc;
+      } else if ( arity == 1 && name == ATOM_set )
+      { int i;
+	if ( !get_int_arg(1, option, &i) ) return false;
+	rc = history(ctx->history, &ev, H_SET, i);
       } else if ( name == ATOM_clear )
       { if ( arity != 0 ) goto err_domain;
 
@@ -1994,6 +2118,130 @@ pl_history(term_t tin, term_t option)
   return FALSE;
 }
 
+#ifndef HAVE_OPEN_MEMSTREAM
+#ifdef HAVE_FUNOPEN
+#define HAVE_OPEN_MEMSTREAM 1
+
+typedef struct
+{ char **bufp;      /* pointer to user buffer pointer */
+  size_t *sizep;    /* pointer to user size variable */
+  char *buffer;     /* allocated buffer */
+  size_t length;    /* bytes written */
+  size_t capacity;  /* allocated size */
+} MEMSTREAM;
+
+static int
+ms_write(void *cookie, const char *buf, int len)
+{ MEMSTREAM *m = (MEMSTREAM *)cookie;
+  size_t needed = m->length + len + 1;  /* +1 for NUL */
+
+  if (needed > m->capacity)
+  { size_t newcap = m->capacity ? m->capacity * 2 : 128;
+    while (newcap < needed) newcap *= 2;
+    char *newbuf = realloc(m->buffer, newcap);
+    if (!newbuf)
+      return -1;
+    m->buffer = newbuf;
+    m->capacity = newcap;
+  }
+
+  memcpy(m->buffer + m->length, buf, len);
+  m->length += len;
+  m->buffer[m->length] = '\0';  /* always null-terminate */
+
+  return len;
+}
+
+static fpos_t
+ms_seek(void *cookie, fpos_t offset, int whence)
+{ MEMSTREAM *m = (MEMSTREAM *)cookie;
+  size_t newpos;
+
+  switch (whence)
+  { case SEEK_SET:
+      newpos = (size_t)offset; break;
+    case SEEK_CUR:
+      newpos = m->length + (size_t)offset; break;
+    case SEEK_END:
+      newpos = m->length + (size_t)offset; break;
+    default: return -1;
+  }
+
+  if (newpos > m->length)
+    newpos = m->length;
+
+  return (fpos_t)newpos;
+}
+
+static int
+ms_close(void *cookie)
+{ MEMSTREAM *m = (MEMSTREAM *)cookie;
+  /* update user pointers */
+  if (m->bufp)  *m->bufp = m->buffer;
+  if (m->sizep) *m->sizep = m->length;
+  free(m);
+  return 0;
+}
+
+FILE *
+open_memstream(char **bufp, size_t *sizep)
+{ MEMSTREAM *m = calloc(1, sizeof(MEMSTREAM));
+  if (!m)
+    return NULL;
+
+  m->bufp = bufp;
+  m->sizep = sizep;
+  m->buffer = NULL;
+  m->capacity = 0;
+  m->length = 0;
+
+  FILE *f = NULL;
+  f = funopen(m, NULL, ms_write, ms_seek, ms_close);
+  if (!f)
+  { free(m);
+    return NULL;
+  }
+
+  return f;
+}
+#endif /*HAVE_FUNOPEN*/
+#endif/*HAVE_OPEN_MEMSTREAM*/
+
+#if defined(HAVE_OPEN_MEMSTREAM) && defined(H_SAVE_FP)
+#define HAVE_EL_HISTORY_ENCODED 1
+static foreign_t
+el_history_encoded(term_t raw, term_t encoded)
+{ char *rs;
+
+  if ( PL_get_chars(raw, &rs, STR_OPTIONS) )
+  { History *hist = history_init();
+    HistEvent ev;
+    char *es = NULL;
+    size_t len = 0;
+    bool rc = false;
+
+    history(hist, &ev, H_SETSIZE, 1);
+    history(hist, &ev, H_ENTER,   rs);
+    FILE *fd = open_memstream(&es, &len);
+    history(hist, &ev, H_SAVE_FP, fd);
+    fclose(fd);
+    if ( es )
+    { char *s = strchr(es, '\n');		/* Skip magic */
+      if ( s )
+      { s++;
+	len -= s-es;
+	rc = PL_unify_chars(encoded, PL_STRING|REP_MB, len, s);
+      }
+    }
+    free(es);
+    history_end(hist);
+    return rc;
+  } else
+    return false;
+}
+#endif
+
+
 static foreign_t
 el_version(term_t version)
 {
@@ -2027,29 +2275,39 @@ install_libedit4pl(void)
   MKATOM(fatal);
   MKATOM(clear);
   MKATOM(setsize);
+  MKATOM(getsize);
   MKATOM(setunique);
+  MKATOM(set);
+  MKATOM(prev_str);
+  MKATOM(next_str);
+  MKATOM(event);
 
+  MKFUNCTOR(error, 2);
+  MKFUNCTOR(editline, 1);
   MKFUNCTOR(line, 2);
   MKFUNCTOR(electric, 3);
   FUNCTOR_pair2 = PL_new_functor(PL_new_atom("-"), 2);
 
-  PL_register_foreign("el_wrap",	  5, pl_wrap,	       0);
-  PL_register_foreign("el_wrapped",	  1, pl_is_wrapped,    0);
-  PL_register_foreign("el_unwrap",	  1, pl_unwrap,	       0);
-  PL_register_foreign("el_source",	  2, pl_source,	       0);
-  PL_register_foreign("el_addfn",	  4, pl_addfn,	       0);
-  PL_register_foreign("el_bind",	  2, pl_bind,	       0);
-  PL_register_foreign("el_cursor",	  2, pl_cursor,	       0);
-  PL_register_foreign("el_line",	  2, pl_line,	       0);
-  PL_register_foreign("el_insertstr",	  2, pl_insertstr,     0);
-  PL_register_foreign("el_deletestr",	  2, pl_deletestr,     0);
-  PL_register_foreign("el_add_history",	  2, pl_add_history,   0);
-  PL_register_foreign("el_write_history", 2, pl_write_history, 0);
-  PL_register_foreign("el_read_history",  2, pl_read_history,  0);
-  PL_register_foreign("el_history_events",2, pl_history_events,0);
-  PL_register_foreign("el_history",       2, pl_history,       0);
-  PL_register_foreign("el_getc",          2, pl_getc,          0);
-  PL_register_foreign("el_push",          2, pl_push,          0);
-  PL_register_foreign("el_editmode",      2, pl_editmode,      0);
-  PL_register_foreign("el_version",       1, el_version,       0);
+  PL_register_foreign("el_wrap",	     5,	pl_wrap,	    0);
+  PL_register_foreign("el_wrapped",	     1,	pl_is_wrapped,	    0);
+  PL_register_foreign("el_unwrap",	     1,	pl_unwrap,	    0);
+  PL_register_foreign("el_source",	     2,	pl_source,	    0);
+  PL_register_foreign("el_addfn",	     4,	pl_addfn,	    0);
+  PL_register_foreign("el_bind",	     2,	pl_bind,	    0);
+  PL_register_foreign("el_cursor",	     2,	pl_cursor,	    0);
+  PL_register_foreign("el_line",	     2,	pl_line,	    0);
+  PL_register_foreign("el_insertstr",	     2,	pl_insertstr,	    0);
+  PL_register_foreign("el_deletestr",	     2,	pl_deletestr,	    0);
+  PL_register_foreign("el_add_history",	     2,	pl_add_history,	    0);
+  PL_register_foreign("el_write_history",    2,	pl_write_history,   0);
+  PL_register_foreign("el_read_history",     2,	pl_read_history,    0);
+  PL_register_foreign("el_history_events",2, pl_history_events,     0);
+  PL_register_foreign("el_history",	     2,	pl_history,	    0);
+  PL_register_foreign("el_getc",	     2,	pl_getc,	    0);
+  PL_register_foreign("el_push",	     2,	pl_push,	    0);
+  PL_register_foreign("el_editmode",	     2,	pl_editmode,	    0);
+  PL_register_foreign("el_version",	     1,	el_version,	    0);
+#ifdef HAVE_EL_HISTORY_ENCODED
+  PL_register_foreign("el_history_encoded",  2,	el_history_encoded, 0);
+#endif
 }
