@@ -185,11 +185,7 @@ openPopup(PopupObj p, Graphical gr, Point pos,
   sw = createPopupWindow(d);
   send(sw, NAME_display, p, EAV);
 
-#ifdef SDL_GRAPHICS
   offset = getFramePositionGraphical(gr);
-#else
-  offset = getDisplayPositionGraphical(gr);
-#endif
   if ( !offset )
     return errorPce(p, NAME_graphicalNotDisplayed, gr);
 
@@ -239,29 +235,6 @@ openPopup(PopupObj p, Graphical gr, Point pos,
     moved = TRUE;
   }
 
-#ifndef SDL_GRAPHICS
-  if ( ensure_on_display == ON )
-  { Monitor mon;			/* Monitor displaying gr */
-    int mx, my, mw, mh;			/* Monitor area */
-
-    if ( (mon=get(gr, NAME_monitor, EAV)) )
-    { mx = valInt(mon->area->x);
-      my = valInt(mon->area->y);
-      mw = valInt(mon->area->w);
-      mh = valInt(mon->area->h);
-    } else				/* Or give error? */
-    { mx = my = 0;
-      mw = valInt(getWidthDisplay(d));
-      mh = valInt(getHeightDisplay(d));
-    }
-
-    if ( px < mx )         moved = TRUE, px = mx;
-    if ( py < my )         moved = TRUE, py = my;
-    if ( px + pw > mw+mx ) moved = TRUE, px = mw+mx - pw;
-    if ( py + ph > mh+my ) moved = TRUE, py = mh+my - ph;
-  }
-#endif
-
   swfr = getFrameGraphical((Graphical) sw);
   fr   = getFrameGraphical(gr);
   if ( fr )
@@ -290,7 +263,6 @@ closePopup(PopupObj p)
     assign(p, pullright, NIL);
   }
 
-#if SDL_GRAPHICS
   FrameObj fr = getFrameGraphical((Graphical)p);
   if ( fr )
   { if ( notNil(p->device) )
@@ -299,15 +271,6 @@ closePopup(PopupObj p)
     }
     send(fr, NAME_destroy, EAV);
   }
-#else
-  PceWindow sw = (PceWindow) p->device;
-  if ( notNil(sw) )
-  { send(sw, NAME_show, OFF, EAV);
-    send(sw, NAME_sensitive, OFF, EAV);
-    send(sw, NAME_clear, EAV);
-    assign(p, displayed, OFF);
-  }
-#endif
 
   succeed;
 }
@@ -351,7 +314,8 @@ executePopup(PopupObj p, Any context)
     if ( instanceOfObject(m, ClassMenu) )
     { if ( notNil(p->selected_item) )
       { selectionMenu(m, p->selected_item);
-	BUSY(forwardMenu(m, m->message, EVENT->value));
+	EventObj ev = getValueVar(EVENT);
+	BUSY(forwardMenu(m, m->message, ev));
       }
     } else
       return errorPce(context, NAME_unexpectedType, ClassMenu);
