@@ -196,16 +196,16 @@ extern install_t	install_pcecall(void);
 		 *	      PROFILER		*
 		 *******************************/
 
-static int prof_active;			/* activated */
+static bool prof_active;		/* activated */
+static bool pce_initialised = false;	/* library is initialiased */
 static PL_prof_type_t pceProfType;	/* registration */
-static int pce_initialised = FALSE;	/* library is initialiased */
 
 		 /*******************************
 		 *	       CONTEXT		*
 		 *******************************/
 
-static Module	 DefaultModule;		/* For module handling */
-static PceObject DefaultContext;	/* Executing context */
+static __thread Module	  DefaultModule;	/* For module handling */
+static __thread PceObject DefaultContext;	/* Executing context */
 
 		 /*******************************
 		 *	     CONSTANTS		*
@@ -612,7 +612,7 @@ install_t
 install_pl2xpce(void)
 { if ( pce_initialised )
     return;
-  pce_initialised = TRUE;
+  pce_initialised = true;
 
   PL_register_foreign("pce_init", 2,
 		      pl_pce_init, PL_FA_TRANSPARENT);
@@ -650,7 +650,7 @@ install_t
 uninstall_pl2xpce(void)
 { if ( !pce_initialised )
     return;
-  pce_initialised = FALSE;
+  pce_initialised = false;
 
   DEBUG(Sdprintf("Removing hooks (%p and %p)\n",
 		 old_dispatch_hook, old_update_hook));
@@ -1945,9 +1945,9 @@ invoke(term_t receiver, term_t class, term_t selector, term_t return)
     don't care.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static int
+static bool
 invoke(term_t rec, term_t cl, term_t msg, term_t ret)
-{ int rval = FALSE;
+{ bool rval = false;
   AnswerMark mark;
   PceObject receiver;
   Module odm;
@@ -2053,7 +2053,7 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 	    if ( i < 0 )
 	    { if ( !PL_unify_list(tail, tmp2, tail) ||
 		   !PL_unify(tmp2, tmp) )
-	      { rval = FALSE;
+	      { rval = false;
 		goto out;
 	      }
 	    } else
@@ -2061,7 +2061,7 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 	  }
 	  if ( tail )
 	  { if ( !PL_unify_nil(tail) )
-	    { rval = FALSE;
+	    { rval = false;
 	      goto out;
 	    }
 	  }
@@ -2073,7 +2073,7 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 
 	  if ( !PL_cons_functor_v(av+1, pcd->functor, mav) ||
 	       !put_object(av+2, goal.receiver) )
-	  { rval = FALSE;
+	  { rval = false;
 	    goto out;
 	  }
 
@@ -2090,7 +2090,7 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 	    if ( rval )
 	    { if ( IsFunctor(av+3, FUNCTOR_ref1) )
 	      { if ( !get_object_from_refterm(av+3, &goal.rval) )
-		{ rval = FALSE;
+		{ rval = false;
 		  goto out;
 		}
 
@@ -2163,6 +2163,9 @@ out:
     PL_close_foreign_frame(fid);
   UNLOCK();
 
+  if ( rval && PL_exception(0) )
+    rval = false;		/* make sure to propagate Prolog exceptions */
+
   return rval;
 }
 
@@ -2272,7 +2275,7 @@ PrologSend(PceObject prolog, PceObject sel, int argc, PceObject *argv)
   int rval;
 
   if ( !pce_initialised )
-    return FALSE;
+    return false;
 
   fid = OpenForeign();
   m = pceContextModule();
@@ -2558,7 +2561,7 @@ PrologCall(PceGoal goal)
 { prolog_call_data *pcd;
 
   if ( !pce_initialised )
-    return FALSE;
+    return false;
 
   if ( (pcd = get_pcd(goal->implementation)) )
   { fid_t fid;
@@ -3188,13 +3191,13 @@ PrologOpenResource(const char *name, const char *rc_class, const char *mode)
 		 *	      PROFILING		*
 		 *******************************/
 
-static int
+static bool
 unify_prof_node(term_t t, void *impl)
 { return unifyObject(t, impl, FALSE);
 }
 
 
-static int
+static bool
 get_prof_node(term_t ref, void **impl)
 { atom_t name;
   size_t arity;
@@ -3203,15 +3206,15 @@ get_prof_node(term_t ref, void **impl)
        name == ATOM_ref &&
        arity == 1 )
   { *impl = termToObject(ref, NULL, NULLATOM, FALSE);
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 
 static void
-prof_activate(int active)
+prof_activate(bool active)
 { pce_profile_hooks hooks;
 
   memset(&hooks, 0, sizeof(hooks));
@@ -3226,7 +3229,7 @@ prof_activate(int active)
 }
 
 static void
-registerProfiler()
+registerProfiler(void)
 { pceProfType.unify    = unify_prof_node;
   pceProfType.get      = get_prof_node;
   pceProfType.activate = prof_activate;
@@ -3263,7 +3266,7 @@ host-language. The non-vararg arguments are in g->argv[], represented as
 Prolog terms.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static int
+static bool
 PrologWriteGoalArgs(PceGoal g)
 { int i, argn = 0;
   term_t l;
@@ -3288,7 +3291,7 @@ PrologWriteGoalArgs(PceGoal g)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -3344,8 +3347,8 @@ do_reset(void)
 }
 
 
-static int
-hasThreadsProlog()
+static bool
+hasThreadsProlog(void)
 { predicate_t pred = PL_predicate("current_prolog_flag", 2, "user");
   term_t av = PL_new_term_refs(2);
 
