@@ -1765,7 +1765,7 @@ PL_cleanup(int status)
   int asked_reclaim_memory = (status&PL_CLEANUP_NO_RECLAIM_MEMORY) == 0;
   int reclaim_memory = asked_reclaim_memory;
 
-  if ( GD->cleaning != CLN_NORMAL )
+  if ( GD->halt.cleaning != CLN_NORMAL )
     return PL_CLEANUP_RECURSIVE;
 
   checkPrologFlagsAccess();
@@ -1776,7 +1776,7 @@ PL_cleanup(int status)
 #endif
 
   PL_LOCK(L_INIT);
-  if ( GD->cleaning != CLN_NORMAL )
+  if ( GD->halt.cleaning != CLN_NORMAL )
   { PL_UNLOCK(L_INIT);
     return PL_CLEANUP_RECURSIVE;
   }
@@ -1792,7 +1792,7 @@ PL_cleanup(int status)
   GET_LD
 #endif
 
-  GD->cleaning = CLN_PROLOG;
+  GD->halt.cleaning = CLN_PROLOG;
   debugmode(false, NULL);		/* avoid recursive tracing */
 
   if ( GD->initialised )
@@ -1804,18 +1804,18 @@ PL_cleanup(int status)
     PL_set_prolog_flag("exit_status", PL_INTEGER, (intptr_t)rval);
     if ( query_loop(PL_new_atom("$run_at_halt"), false) == false &&
 	 !(status&PL_CLEANUP_NO_CANCEL) )
-    { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
-      { GD->cleaning = CLN_NORMAL;
+    { if ( ++GD->halt.cancelled	< MAX_HALT_CANCELLED )
+      { GD->halt.cleaning = CLN_NORMAL;
 	PL_UNLOCK(L_INIT);
 	return PL_CLEANUP_CANCELED;
       }
     }
 
-    GD->cleaning = CLN_FOREIGN;
+    GD->halt.cleaning = CLN_FOREIGN;
     if ( !run_on_halt(&GD->os.on_halt_list, rval) &&
 	 !(status&PL_CLEANUP_NO_CANCEL) )
-    { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
-      { GD->cleaning = CLN_NORMAL;
+    { if ( ++GD->halt.cancelled	< MAX_HALT_CANCELLED )
+      { GD->halt.cleaning = CLN_NORMAL;
 	PL_UNLOCK(L_INIT);
 	return PL_CLEANUP_CANCELED;
       }
@@ -1838,7 +1838,7 @@ PL_cleanup(int status)
 
 emergency:
 #endif
-  GD->cleaning = CLN_IO;
+  GD->halt.cleaning = CLN_IO;
 
   Scurout = Soutput;			/* reset output stream to user */
 
@@ -1846,7 +1846,7 @@ emergency:
   dieIO();				/* streams may refer to foreign code */
 					/* Standard I/O is only flushed! */
 
-  GD->cleaning = CLN_SHARED;
+  GD->halt.cleaning = CLN_SHARED;
 
   if ( GD->initialised )
   { fid_t cid = PL_open_foreign_frame();
@@ -1857,7 +1857,7 @@ emergency:
     PL_discard_foreign_frame(cid);
   }
 
-  GD->cleaning = CLN_DATA;
+  GD->halt.cleaning = CLN_DATA;
 
   RemoveTemporaryFiles();
 
@@ -2256,7 +2256,7 @@ static
 PRED_IMPL("$run_state", 1, run_state, 0)
 { const char *rstate;
 
-  switch(GD->cleaning)
+  switch(GD->halt.cleaning)
   { case CLN_NORMAL:
       rstate = "normal";
       break;
