@@ -39,17 +39,17 @@
 
 
 		/********************************
-		*	     VARIANT	        *
+		*	     VARIANT		*
 		*********************************/
 
-#define consVar(w) (((intptr_t)(w)<<LMASK_BITS) | TAG_VAR | FIRST_MASK)
-#define valVar(w)  ((intptr_t)(w) >> LMASK_BITS)
+#define consVar(w) (((word)(w)<<LMASK_BITS) | TAG_VAR | FIRST_MASK)
+#define valVar(w)  ((size_t)((word)(w) >> LMASK_BITS))
 
 #define TAG_COMPOUND_x    (STG_STATIC|TAG_COMPOUND)
 #define isCompound_x(w) \
-  (( (intptr_t)(w) & (STG_MASK|TAG_MASK) ) == TAG_COMPOUND_x )
-#define valCompound_x(w)  ((intptr_t)(w) >> LMASK_BITS)
-#define consCompound_x(i)  (((i)<<LMASK_BITS) | TAG_COMPOUND_x)
+	(( (word)(w) & (STG_MASK|TAG_MASK) ) == TAG_COMPOUND_x )
+#define valCompound_x(w)  ((size_t)((word)(w) >> LMASK_BITS))
+#define consCompound_x(i) (((word)(i)<<LMASK_BITS) | TAG_COMPOUND_x)
 
 #define node_bp(p)	((p)->bp)
 #define node_orig(p)	((p)->orig)
@@ -57,10 +57,10 @@
 #define node_isom(p)	((p)->b)
 
 typedef struct aWork
-{ word *	left;			/* left term arguments */
-  word *	right;			/* right term arguments */
-  int		arg;
-  int		arity;
+{ Word		left;			/* left term arguments */
+  Word		right;			/* right term arguments */
+  size_t	arg;
+  size_t	arity;
 } aWork;
 
 
@@ -87,19 +87,19 @@ init_agenda(argPairs *a)
 
 
 static inline bool
-push_start_args(argPairs *a, Word left, Word right, int arity)  /* plural */
+push_start_args(argPairs *a, Word left, Word right, size_t arity)  /* plural */
 { a->work.left  = left;
   a->work.right = right;
   a->work.arg   = 0;
   a->work.arity = arity;
 
-  return TRUE;
+  return true;
 }
 
 static inline bool
-push_args(argPairs *a, Word left, Word right, int arity)  /* plural */
+push_args(argPairs *a, Word left, Word right, size_t arity)  /* plural */
 { if ( !pushSegStack(&a->stack, a->work, aWork) )
-    return FALSE;
+    return false;
 
   return push_start_args(a, left, right, arity);
 }
@@ -108,7 +108,7 @@ static inline bool
 next_arg(argPairs *a, Word *lp, Word *rp)   /* singular (not plural !) */
 { while( a->work.arg >= a->work.arity)
   { if ( !popSegStack(&a->stack, &a->work, aWork) )
-      return FALSE;
+      return false;
   }
 
   *lp = a->work.left;
@@ -118,11 +118,11 @@ next_arg(argPairs *a, Word *lp, Word *rp)   /* singular (not plural !) */
   a->work.left++;
   a->work.right++;
 
-  return TRUE;
+  return true;
 }
 
 static inline node *
-Node(int i, Buffer buf)
+Node(size_t i, Buffer buf)
 { return  &fetchBuffer(buf, i, node);
 }
 
@@ -130,47 +130,47 @@ static inline bool
 add_node_buffer(Buffer b, node *obj)
 { if ( ((b->top) + sizeof(node)) > (b->max) )
   { if ( !growBuffer(b, sizeof(node)) )
-      return FALSE;
+      return false;
   }
 
   *((node *)(b)->top) = *obj;
   b->top += sizeof(node);
 
-  return TRUE;
+  return true;
 }
 
-static inline int
+static size_t
 var_id(Word p, Buffer buf)
 { word w = *p;
 
-  if ( w )
-  { return (int)valVar(w);		/* node id truncated to int: */
+  if ( (w&FIRST_MASK) )
+  { return valVar(w);		/* node id truncated to int: */
   } else				/* < 2^31 nodes */
-  { int n = (int)entriesBuffer(buf, node);
+  { size_t n = entriesBuffer(buf, node);
     node new = {p, w, 0, 0};
 
     if ( !add_node_buffer((Buffer)buf, &new) )
       return MEMORY_OVERFLOW;
 
-    *p = (word)consVar(n);
+    *p = consVar(n);
     return n;
   }
 }
 
-static inline int
+static size_t
 term_id(Word p, Buffer buf)
 { word w = *p;
 
   if ( isCompound_x(w) )
-  { return (int)valCompound_x(w);
+  { return valCompound_x(w);
   } else
-  { int n = (int)entriesBuffer(buf, node);
+  { size_t n = entriesBuffer(buf, node);
     node new = {p, w, 0, 0};
 
     if ( !add_node_buffer((Buffer)buf, &new) )
       return MEMORY_OVERFLOW;
 
-    *p = (word)consCompound_x(n);
+    *p = consCompound_x(n);
     return n;
   }
 }
@@ -192,8 +192,9 @@ Root(int i, node **r, Buffer buf)
   return k;
 }
 
+#define univ(t, d, a) LDFUNC(univ, t, d, a)
 static inline void
-univ(word t, Word d, Word *a ARG_LD)
+univ(DECL_LD word t, Word d, Word *a)
 { Functor f;
 
   f = valueTerm(t);
@@ -204,37 +205,39 @@ univ(word t, Word d, Word *a ARG_LD)
 }
 
 static inline void
-reset_terms(node * r)
-{ *(r->bp)  =  r->orig;
+reset_terms(node *r)
+{ IS_WORD_ALIGNED(r->bp);
+  *r->bp = r->orig;
 }
 
 /* isomorphic (==) */
 
+#define isomorphic(a, i, j, buf) LDFUNC(isomorphic, a, i, j, buf)
 static int
-isomorphic(argPairs *a, int i, int j, Buffer buf ARG_LD)
+isomorphic(DECL_LD argPairs *a, int i, int j, Buffer buf)
 { Word l = NULL, r = NULL, lm, ln;
   word dm, dn;
   Word dummy = NULL;
 
   if ( i == j )
-    return TRUE;
+    return true;
 
   if ( !push_args(a, dummy, dummy, 1) )
     return MEMORY_OVERFLOW;
 
-  univ(node_orig(Node(i, buf)), &dm, &lm PASS_LD);
-  univ(node_orig(Node(j, buf)), &dn, &ln PASS_LD);
+  univ(node_orig(Node(i, buf)), &dm, &lm);
+  univ(node_orig(Node(j, buf)), &dn, &ln);
 
   if ( dm != dn )
-    return FALSE;
+    return false;
 
-  if ( !push_args(a,  lm, ln, arityFunctor(dm)) )
+  if ( !push_args(a, lm, ln, arityFunctor(dm)) )
     return MEMORY_OVERFLOW;
 
   while( next_arg(a, &l, &r) )
   { word wl, wr;
     if ( l == NULL )
-      return TRUE;
+      return true;
 
   attvar:
     deRef(l);
@@ -244,11 +247,11 @@ isomorphic(argPairs *a, int i, int j, Buffer buf ARG_LD)
     wr = *r;
 
     if ( tag(wl) != tag(wr) )
-      return FALSE;
+      return false;
 
     if ( tag(wl) == TAG_VAR )
     { if ( l != r )		/* identity test on variables */
-	return FALSE;
+	return false;
       continue;
     }
 
@@ -263,20 +266,20 @@ isomorphic(argPairs *a, int i, int j, Buffer buf ARG_LD)
 
     switch(tag(wl))
     { case TAG_ATOM:
-	return FALSE;
+	return false;
       case TAG_INTEGER:
 	if ( storage(wl) == STG_INLINE ||
 	     storage(wr) == STG_INLINE )
-	  return FALSE;
+	  return false;
       case TAG_STRING:
       case TAG_FLOAT:
 	if ( equalIndirect(wl, wr) )
 	  continue;
-        return FALSE;
+        return false;
       case TAG_COMPOUND:
       { Word lm, ln;
 	word dm, dn;
-	int i, j;
+	size_t i, j;
 	node  *m,  *n;
 
 	if ( (i = term_id(l, buf)) < 0 )
@@ -290,11 +293,11 @@ isomorphic(argPairs *a, int i, int j, Buffer buf ARG_LD)
 	if ( i==j )
 	  continue;
 
-	univ(node_orig(m), &dm, &lm PASS_LD);
-	univ(node_orig(n), &dn, &ln PASS_LD);
+	univ(node_orig(m), &dm, &lm);
+	univ(node_orig(n), &dn, &ln);
 
 	if ( dm != dn )
-	  return FALSE;
+	  return false;
 
 	if ( i <= j )
 	  node_isom(m) = j;		/* union */
@@ -311,30 +314,44 @@ isomorphic(argPairs *a, int i, int j, Buffer buf ARG_LD)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 /* t =@= u */
+/* returns true, false or MEMORY_OVERFLOW */
+
+#define variant(agenda, buf) LDFUNC(variant, agenda, buf)
 static int
-variant(argPairs *agenda, Buffer buf ARG_LD)
+variant(DECL_LD argPairs *agenda, Buffer buf)
 { Word l = NULL, r =NULL;
 
   while( next_arg(agenda, &l, &r) )
   { word wl, wr;
 
  attvar:
-   deRef(l);
-   deRef(r);
+    IS_WORD_ALIGNED(l);
+    IS_WORD_ALIGNED(r);
+    deRef(l);
+    deRef(r);
 
-   wl = *l;
-   wr = *r;
+    wl = *l;
+    wr = *r;
 
-   if ( tag(wl) != tag(wr) )
-     return FALSE;
+    if ( tag(wl) != tag(wr) )
+      return false;
 
-   if ( tag(wl) == TAG_VAR )
-   { int i, j, m, n;
+   if ( needsRef(wl) )		/* var or attvar */
+   { size_t i, j;
+     int m, n;
      node *vl, *vr;
+     Word al, ar;
+
+     if ( tag(wl) == TAG_ATTVAR )
+     { al = valPAttVar(wl);
+       ar = valPAttVar(wr);
+     } else
+     { al = ar = NULL;
+     }
 
      if ((i = var_id(l, buf)) < 0)
        return MEMORY_OVERFLOW;
@@ -350,19 +367,22 @@ variant(argPairs *agenda, Buffer buf ARG_LD)
      if ( (m==0) && (n==0) )
      { vl->a = j;
        vr->b = i;
+       if ( al )
+       { l = al; r = ar;
+	 goto attvar;
+       }
        continue;
      }
      if ( (m != 0) && (n != 0) )
      { if ( (m == j) && (n == i) )
+       { if ( al )
+	 { l = al; r = ar;
+	   goto attvar;
+	 }
 	 continue;
+       }
      }
-     return FALSE;
-    }
-
-    if ( tag(wl) == TAG_ATTVAR )
-    { l = valPAttVar(wl);
-      r = valPAttVar(wr);
-      goto attvar;
+     return false;
     }
 
     if ( wl == wr && !isTerm(wl) )
@@ -370,18 +390,19 @@ variant(argPairs *agenda, Buffer buf ARG_LD)
 
     switch(tag(wl))
     { case TAG_ATOM:
-	return FALSE;
+	return false;
       case TAG_INTEGER:
 	if ( storage(wl) == STG_INLINE ||
 	     storage(wr) == STG_INLINE )
-	  return FALSE;
+	  return false;
       case TAG_STRING:
       case TAG_FLOAT:
 	if ( equalIndirect(wl, wr) )
 	  continue;
-        return FALSE;
+        return false;
       case TAG_COMPOUND:
-	{ int i, j, k, h;
+      {   size_t i, j;
+	  int k, h;
 	  node *m;
 
 	  word dm, dn;			/* definition (= functor/arity) */
@@ -396,16 +417,16 @@ variant(argPairs *agenda, Buffer buf ARG_LD)
 	  k = node_variant(m);
 
 	  if ( 0 != k )
-	  { if ( ( h = isomorphic(agenda, k, j, buf PASS_LD) ) <= 0 )
+	  { if ( ( h = isomorphic(agenda, k, j, buf) ) <= 0 )
 	      return  h;
 	    continue;
 	  }
 
-	  univ(node_orig(m), &dm, &lm PASS_LD);
-	  univ(node_orig(Node(j,buf)), &dn, &ln PASS_LD);
+	  univ(node_orig(m), &dm, &lm);
+	  univ(node_orig(Node(j,buf)), &dn, &ln);
 
 	  if ( dm != dn )
-	    return FALSE;
+	    return false;
 
 	  node_variant(m) = j;
 
@@ -417,16 +438,17 @@ variant(argPairs *agenda, Buffer buf ARG_LD)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
+/* returns true, false or false+exception */
 int
-is_variant_ptr(Word p1, Word p2 ARG_LD)
+is_variant_ptr(DECL_LD Word p1, Word p2)
 { argPairs agenda;
   tmp_buffer buf;
   Buffer VARIANT_BUFFER = (Buffer)&buf;
-  bool rval;
+  int rval;
   node *r;
   node new = {NULL, 0, 0, 0};   /* dummy node as 0-th element*/
 
@@ -434,22 +456,22 @@ is_variant_ptr(Word p1, Word p2 ARG_LD)
   deRef(p2);
 
   if ( *p1 == *p2 )                     /* same term */
-    return TRUE;
+    return true;
   if ( tag(*p1) != tag(*p2) )           /* different type */
-    return FALSE;
+    return false;
 again:
   switch(tag(*p1))                      /* quick tests */
   { case TAG_VAR:
-      return TRUE;
+      return true;
     case TAG_ATTVAR:
       p1 = valPAttVar(*p1);
       p2 = valPAttVar(*p2);
       goto again;
     case TAG_ATOM:
-      return FALSE;
+      return false;
     case TAG_INTEGER:
       if ( !(isIndirect(*p1) && isIndirect(*p2)) )
-        return FALSE;
+        return false;
       /*FALLTHROUGH*/
     case TAG_FLOAT:
     case TAG_STRING:
@@ -459,21 +481,20 @@ again:
       Functor t2 = valueTerm(*p2);
 
       if ( t1->definition != t2->definition )
-        return FALSE;
+        return false;
       break;
     }
     default:
       assert(0);
-      return FALSE;
+      return false;
   }
 
-  startCritical;
   initBuffer(&buf);	/* can be faster! */
   init_agenda(&agenda);
 
-  if ( (add_node_buffer(VARIANT_BUFFER, &new) >= 0) &&
-       (push_start_args(&agenda, p1, p2, 1) >=0) )
-    rval = variant(&agenda, VARIANT_BUFFER PASS_LD);
+  if ( add_node_buffer(VARIANT_BUFFER, &new) &&
+       push_start_args(&agenda, p1, p2, 1) )
+    rval = variant(&agenda, VARIANT_BUFFER);
   else
     rval = MEMORY_OVERFLOW;
 
@@ -486,9 +507,6 @@ again:
 
   DEBUG(CHK_SECURE, checkStacks(NULL));
 
-  if ( !endCritical )
-    return FALSE;
-
   if ( rval >= 0 )
     return rval;
 
@@ -499,14 +517,14 @@ static
 PRED_IMPL("=@=", 2, variant, 0)
 { PRED_LD
 
-  return is_variant_ptr(valTermRef(A1), valTermRef(A2) PASS_LD);
+  return is_variant_ptr(valTermRef(A1), valTermRef(A2));
 }
 
 static
 PRED_IMPL("\\=@=", 2, not_variant, 0)
 { PRED_LD
 
-  return !is_variant_ptr(valTermRef(A1), valTermRef(A2) PASS_LD);
+  return !is_variant_ptr(valTermRef(A1), valTermRef(A2));
 }
 
 		 /*******************************
